unit global;

interface

uses
     sysutils, forms, classes, dialogs, inifiles, messages, fileutil, LConvEncoding, sqldb;

type
     tformat = (_VCD, _SVCD, _MVCD, _MSVCD, _AVIDivX, _DVD, _Anderes);

     tmarker    = array [1 .. 10] of boolean;
     tmarkerbez = array [1 .. 10] of shortstring;

     trec = record
          ID        :    integer;
          titel     :    string [100];
          Datum     :    tdatetime;
          Format    :    string [015];
          Jahr      :    string [004];
          Genre     :    string [040];
          Bemerkung :    string [080];
          Darsteller:    string [239];
          Geloescht :    boolean;
          Infosize  :    integer;
          info      :    ansistring;
          Groesse   :    int64;
          Gesehen   :    Boolean;
          Marker    :    tmarker;
          HDName    :    string [015];
          Verz      :    string [120];
     end;

     trecs = packed record  // immer 5.000 byte    trecS > S für speichern wegen der Ansistrings
          ID        :    integer;          //   4 Byte     4
          titel     :    string [100];     // 101        105
          Datum     :    tdatetime;        //   8        113
          Format    :    string [015];     //  16        129
          Jahr      :    string [004];     //   5        134
          Genre     :    string [040];     //  41        175
          Bemerkung :    string [080];     //  81        256
          Darsteller:    string [239];     // 240        496
          Geloescht :    boolean;          //   1        497
          Infosize  :    integer;          //   4        501
          Info      :    array [1 .. 4000] of char; // 4.501
          Groesse   :    int64;            //   8      4.509
          Gesehen   :    Boolean;          //   1      4.510
          Marker    :    tmarker;          //  10      4.520
          HDName    :    string [015];     //  16      4.536
          Verz      :    string [120];     // 121      4.657
          dummy     :    array [1 .. 0343] of byte; // Platzhalter bis 5.000
     end;

     tmodus = (manuell, automatisch);

     tautomatisch = record
          modus : tmodus;
          lwname,
          verzname,
          txtname : shortstring;
     end;

     tlisten = record
          titel,
          genre,
          jahr,
          format,
          datum,
          cdnr,
          stempel,
          detail    : boolean;
          dateiname,
          verz      : string;
          namen     : tstringlist;
     end;

     tus = record
          HFVollbild,
          EFVollbild,
          IFVollbild     : boolean;
          HFPos,
          EFPos,
          IFPos          : trect;
          Listen         : TListen;
          Sicherungsverz : ansistring;
          Datensicherung : boolean;
          Markerbez      : TMarkerbez;
          ProgTitel      : string [50];
          AktHDName      : string [15];
     end;

const
     formatstr : array [tformat] of string = ('VCD', 'SVCD', 'MVCD', 'MSVCD', 'AVI/DivX', 'DVD', 'Anderes');

var
     frec      : file of trecs;
     rec       : trec;
     recs      : trecs;
     loeschanz : integer;
     j, k      : integer;
     progverz,
     filmverz,
     bildverz,
     bioverz   : ansistring;
     ininame   : ansistring;
     index     : ansistring;
     dbname    : ansistring;
     txtverz   : ansistring;
     sb        : ansistring;
     auto      : tautomatisch;
     sort      : array of integer;
     hilfe     : array of string;
     sortiert  : (sID, sTitel, sGenre, sJahr, sFormat, sDatum, sHDName);
     us        : tus;
     funktion  : (ff_hinzufuegen, ff_korrigieren);

const
     id : string = '';
     erweiterte_suche : boolean = false;
     nur_geloeschte : boolean = false;
     Marker : 0 .. 10 = 0;
     sicherung_notwendig : boolean = false;
     nina   : boolean = false;
     twiggy : boolean = false;

function  bild_vorhanden (darsteller : ansistring) : boolean;
function  biografie_vorhanden (darsteller : ansistring) : boolean;
procedure darsteller_aufbereiten (darsteller : ansistring; liste : tstrings);
function  dbdatum (datum : string) : string;
function  filmografie_vorhanden (darsteller : ansistring) : boolean;
procedure lesen (id : integer);
function  recordcount : integer;
procedure schreiben (id : integer);
procedure umgebung_lesen;
procedure umgebung_schreiben;



implementation




uses
     UFHaupt, UFHinzufuegen;

var
     letzte_datei : boolean;




function  bild_vorhanden (darsteller : ansistring) : boolean;

begin
     result := fileexists (bildverz + darsteller + '.jpg');
end;




function  biografie_vorhanden (darsteller : ansistring) : boolean;

begin
     result := fileexists (bioverz + darsteller + '.txt');
end;




procedure darsteller_aufbereiten (darsteller : ansistring; liste : tstrings);

var
     j : integer;
     h, s : ansistring;

begin
     liste.Clear;

     h := '';
     s := darsteller;
     j := 1;

     while j <= length (s) do
     begin
          if   s [j] = ';' then
          begin
               if   length (h) > 0
               then liste.add (trim (h));

               h := '';
               inc (j, 2);
          end else
          begin
               h := h + s [j];
               inc (j);
          end;
     end;

     if   length (h) > 0
     then liste.add (trim (h));
end;




function  dbdatum (datum : string) : string;

var
     s, h : string;

begin
     result := '';

     if   length (datum) <> 10
     then exit;

     result := copy (datum, 7, 4) + '-' + copy (datum, 4, 2) + '-' + copy (datum, 1,2);
end;




function  filmografie_vorhanden (darsteller : ansistring) : boolean;

begin
     result := fileexists (filmverz + darsteller + '.dat');
end;




procedure lesen (id : integer);

var
     g, k : integer;

begin
     g := system.filesize (frec);

     if   id > g then
     begin
          showmessage ('Lesen hinter Dateiende ! ' + inttostr (id) + ' / ' + inttostr (system.filesize (frec)));
     end;

     seek (frec, pred (id));
     read (frec, recs);

     if   recs.infosize  > sizeof (recs.info)
     then recs.infosize := sizeof (recs.info);

     recs.id        := id;

     rec.ID         := recs.ID;
     rec.titel      := recs.titel;
     rec.Datum      := recs.Datum;
     rec.Format     := recs.Format;
     rec.Jahr       := recs.Jahr;
     rec.Genre      := recs.Genre;
     rec.Bemerkung  := recs.Bemerkung;
     rec.Darsteller := recs.Darsteller;
     rec.geloescht  := recs.geloescht;
     rec.Infosize   := recs.Infosize;

     rec.Info := '';

     for  k := 1 to recs.Infosize
     do   rec.Info := rec.info + recs.info [k];

     rec.info       := rec.info;
     rec.Groesse    := recs.Groesse;
     rec.gesehen    := recs.gesehen;
     rec.marker     := recs.marker;
     rec.HDName     := recs.HDName;
     rec.Verz       := recs.Verz;
end;




procedure schreiben (id : integer);

var
     k : integer;

begin
     fillchar (recs, sizeof (recs), 0);

     recs.ID         := rec.ID;
     recs.titel      := rec.titel;
     recs.Datum      := rec.Datum;
     recs.Format     := rec.Format;
     recs.Jahr       := rec.Jahr;
     recs.Genre      := rec.Genre;
     recs.Bemerkung  := rec.Bemerkung;
     recs.Darsteller := rec.Darsteller;
     recs.geloescht  := rec.geloescht;
     recs.Infosize   := rec.Infosize;

     for  k := 1 to rec.infosize
     do   recs.Info [k] := rec.info [k];

     recs.HDName     := rec.HDName;
     recs.gesehen    := rec.gesehen;
     recs.marker     := rec.marker;

     recs.Groesse    := rec.groesse;
     recs.verz       := rec.verz;

     seek  (frec, pred (id));
     write (frec, recs);
end;




function recordcount : integer;

begin
     result := high (sort);
end;




procedure umgebung_lesen;

var
     ini : tinifile;
     h : string;
     j, anz : integer;

begin
     ini := tinifile.Create (ininame);

     us.HFPos.left            := ini.ReadInteger ('Fensterpositionen', 'HFLinks'   , 000);
     us.HFPos.top             := ini.readInteger ('Fensterpositionen', 'HFOben'    , 000);
     us.HFPos.Right           := ini.readinteger ('Fensterpositionen', 'HFRechts'  , 640);
     us.HFPos.bottom          := ini.ReadInteger ('Fensterpositionen', 'HFUnten'   , 480);
     us.HFVollbild            := ini.ReadBool    ('Fensterpositionen', 'HFVollbild', false);

     us.EFPos.left            := ini.ReadInteger ('Fensterpositionen', 'EFLinks'   , 000);
     us.EFPos.top             := ini.readInteger ('Fensterpositionen', 'EFOben'    , 000);
     us.EFPos.Right           := ini.readinteger ('Fensterpositionen', 'EFRechts'  , 640);
     us.EFPos.bottom          := ini.ReadInteger ('Fensterpositionen', 'EFUnten'   , 480);
     us.EFVollbild            := ini.ReadBool    ('Fensterpositionen', 'EFVollbild', false);

     us.IFPos.left            := ini.ReadInteger ('Fensterpositionen', 'IFLinks'   , 000);
     us.IFPos.top             := ini.readInteger ('Fensterpositionen', 'IFOben'    , 000);
     us.IFPos.Right           := ini.readinteger ('Fensterpositionen', 'IFRechts'  , 640);
     us.IFPos.bottom          := ini.ReadInteger ('Fensterpositionen', 'IFUnten'   , 480);
     us.IFVollbild            := ini.ReadBool    ('Fensterpositionen', 'IFVollbild', false);

     us.Sicherungsverz        := ini.readstring  ('Datensicherung'   , 'Ordner'    , progverz);
     us.Datensicherung        := ini.readbool    ('Datensicherung'   , 'aktiviert' , false);

     us.listen.titel          := ini.readbool    ('Listen', 'Titel'  , true);
     us.listen.genre          := ini.readbool    ('Listen', 'Genre'  , false);
     us.listen.jahr           := ini.readbool    ('Listen', 'Jahr'   , false);
     us.listen.format         := ini.readbool    ('Listen', 'Format' , false);
     us.listen.datum          := ini.readbool    ('Listen', 'Datum'  , false);
     us.listen.cdnr           := ini.readbool    ('Listen', 'CD-Nr.' , false);
     us.listen.stempel        := ini.readbool    ('Listen', 'Stempel', false);
     us.listen.detail         := ini.readbool    ('Listen', 'Detail' , false);
     us.listen.verz           := ini.readstring  ('Listen', 'Ordner' , '');
     us.listen.dateiname      := ini.readstring  ('Listen', 'Dateiname', '');

     anz                      := ini.readinteger ('Listen', 'Anzahl' , 0);
     us.ProgTitel             := ini.readstring  ('Programmtitel', 'Name', '');
     us.AktHDName             := ini.readstring  ('Letzter HDName', 'HDName', '');
     us.listen.namen.Clear;

     for  j := 1 to anz do
     begin
          h := ini.readstring ('Dateinamen', 'Datei' + inttostr (j), '');
          us.listen.namen.Add (h);
     end;

     for  j := 1 to 10
     do   us.markerbez [j] := ini.readstring ('Markerbedeutungen', 'Marker' + inttostr (j), '');

     ini.free;
end;

procedure umgebung_schreiben;

var
     h : string;
     j, anz : integer;
     ini : tinifile;

begin
     ini := tinifile.Create (ininame);

     ini.writeInteger ('Fensterpositionen', 'HFLinks'   , us.HFPos.left);
     ini.writeInteger ('Fensterpositionen', 'HFOben'    , us.HFPos.top);
     ini.writeinteger ('Fensterpositionen', 'HFRechts'  , us.HFPos.Right);
     ini.writeInteger ('Fensterpositionen', 'HFUnten'   , us.HFPos.bottom);
     ini.writeBool    ('Fensterpositionen', 'HFVollbild', us.HFVollbild);

     ini.writeInteger ('Fensterpositionen', 'EFLinks'   , us.EFPos.left);
     ini.writeInteger ('Fensterpositionen', 'EFOben'    , us.EFPos.top);
     ini.writeinteger ('Fensterpositionen', 'EFRechts'  , us.EFPos.Right);
     ini.writeInteger ('Fensterpositionen', 'EFUnten'   , us.EFPos.bottom);
     ini.writeBool    ('Fensterpositionen', 'EFVollbild', us.EFVollbild);

     ini.writeInteger ('Fensterpositionen', 'IFLinks'   , us.IFPos.left);
     ini.writeInteger ('Fensterpositionen', 'IFOben'    , us.IFPos.top);
     ini.writeinteger ('Fensterpositionen', 'IFRechts'  , us.IFPos.Right);
     ini.writeInteger ('Fensterpositionen', 'IFUnten'   , us.IFPos.bottom);
     ini.writeBool    ('Fensterpositionen', 'IFVollbild', us.IFVollbild);

     ini.writestring  ('Datensicherung'   , 'Ordner'    , us.Sicherungsverz);
     ini.writebool    ('Datensicherung'   , 'aktiviert' , us.datensicherung);

     ini.writebool    ('Listen', 'Titel'  , us.listen.titel  );
     ini.writebool    ('Listen', 'Genre'  , us.listen.genre  );
     ini.writebool    ('Listen', 'Jahr'   , us.listen.jahr   );
     ini.writebool    ('Listen', 'Format' , us.listen.format );
     ini.writebool    ('Listen', 'Datum'  , us.listen.datum  );
     ini.writebool    ('Listen', 'CD-Nr.' , us.listen.cdnr);
     ini.writebool    ('Listen', 'Stempel', us.listen.stempel);
     ini.writebool    ('Listen', 'Detail' , us.listen.detail);
     ini.writestring  ('Listen', 'Ordner' , us.listen.verz);
     ini.writestring  ('Listen', 'Dateiname', us.listen.dateiname);

     ini.writeinteger ('Listen', 'Anzahl' , us.listen.namen.Count);
     ini.writestring  ('Programmtitel', 'Name', us.ProgTitel);
     anz := us.listen.namen.count;

     for  j := 1 to anz do
     begin
          h := us.listen.namen [pred (j)];
          ini.writestring ('Dateinamen', 'Datei' + inttostr (j), h);
     end;

     for  j := 1 to 10
     do   ini.writestring ('Markerbedeutungen', 'Marker' + inttostr (j), us.markerbez [j]);

     ini.free;
end;



(*
procedure konvertieren (id : integer);

var
     j, k : integer;
     f : file of rec;

begin
     assignfile (f, progverz + 'filmdb.iso');
     reset (f);

     read (frec, recs);

     if   recs.infosize  > sizeof (recs.info)
     then recs.infosize := sizeof (recs.info);

     recs.id        := id;

     if   utf8convertierung_erforderlich then
     begin
          rec.ID         := recs.ID;
          rec.titel      := ISO_8859_1ToUTF8 (recs.titel);
          rec.Datum      := recs.Datum;
          rec.Format     := recs.Format;
          rec.Jahr       := recs.Jahr;
          rec.Genre      := ISO_8859_1ToUTF8 (recs.Genre);
          rec.Bemerkung  := ISO_8859_1ToUTF8 (recs.Bemerkung);
          rec.Darsteller := ISO_8859_1ToUTF8 (recs.Darsteller);
          rec.geloescht  := recs.geloescht;
          rec.Infosize   := recs.Infosize;
     end
     else
     begin;
          rec.ID         := recs.ID;
          rec.titel      := recs.titel;
          rec.Datum      := recs.Datum;
          rec.Format     := recs.Format;
          rec.Jahr       := recs.Jahr;
          rec.Genre      := recs.Genre;
          rec.Bemerkung  := recs.Bemerkung;
          rec.Darsteller := recs.Darsteller;
          rec.geloescht  := recs.geloescht;
          rec.Infosize   := recs.Infosize;
     end;

     rec.Info := '';

     for  k := 1 to recs.Infosize
     do   rec.Info := rec.info + recs.info [k];

     rec.info := ISO_8859_1ToUTF8 (rec.info);
     rec.Groesse    := recs.Groesse;
     rec.gesehen    := recs.gesehen;
     rec.marker     := recs.marker;
     rec.HDName     := recs.HDName;
     rec.Verz       := recs.Verz;
end;
*)



initialization

begin
     progverz := {verz_mit_slash }(extractfilepath (application.exename));
     filmverz := progverz + 'Filmografien';
     bioverz  := progverz + 'Biografien';
     bildverz := progverz + 'Bilder';
     ininame  := progverz + 'FilmDB.ini';
     dbname   := progverz + 'FilmDB.dat';
     us.listen.namen := tstringlist.Create;

{     if   not verz_exist (bildverz)
     then mkdir (bildverz);

     if   not verz_exist (filmverz)
     then mkdir (filmverz);

     if   not verz_exist (bioverz)
     then mkdir (bioverz);
}
     umgebung_lesen;
end;

finalization

begin
     umgebung_schreiben;
     us.listen.namen.free;

     if   sicherung_notwendig
     and  us.datensicherung then
     begin
          // Prüfen, ob progverz und Kopieverz identisch sind
          if   progverz = us.sicherungsverz
          then copyfile (pchar (dbname), pchar ({verz_mit_slash }(us.sicherungsverz) + 'Kopie von FilmDB.dat'), false)
          else copyfile (pchar (dbname), pchar ({verz_mit_slash }(us.sicherungsverz) + 'FilmDB.dat'), false);
     end;
end;

end.

